/***************************************************************************/
/*                                                                         */
/*  WRDisp.c                                                               */
/*             Dataanzeige fr Fronius Wechselrichter mit LCD Anzeige      */
/*             PIC16F84  4Mhz   und 868,350Mhz Empfnger                   */
/*                                                                         */
/*      Mode : HD44780 type LCD displays                                   */
/*                                                                         */
/* Interface :     SEL = Port B 4                                          */
/*                  RS = Port B 5                                          */
/*              Data_7 = Port B 3                                          */
/*              Data_6 = Port B 2                                          */
/*              Data_5 = Port B 1                                          */
/*              Data_4 = Port B 0                                          */
/*             DATA_IN = Port A 0                                          */
/*                                                                         */
/* (c) Matthias Stehr 20.01.2002                                           */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
#include <16F84.h>

#use delay(clock=4000000)          // Crystal on the board

#use rs232(baud=1200, rcv=PIN_A0)

#include <lcd_b.c>    // LCD an Port B

#define INIT 0x11
#define ACPB 0x01
#define SUB  0x02
#define NUB  0x03
#define TB   0x04
#define WRSB 0x05
#define WW   0x06
#define FB   0xFF


unsigned int mb,acp,su,nu,t,cs,csown,cmb,dat1,dat2;  // 8 Bit  unsigned
unsigned int wrstatus;
long p,tt,u;                     // 16 Bit unsigned
long w;
unsigned int wh,wl;


/**************************************************************************/
/*                                                                        */
/**************************************************************************/
void main()
{
   acp = 0;
   p = 0;
   u = 0;
   w = 0;
   t = 0;
   wl= 0;
   wh= 0;
   
   lcd_init();
   printf(lcd_putc,"\fInit...");
   delay_ms(1000);
   do
   {
      do
      {
         mb  = getc();
      }
      while(mb != INIT);  // Sync to CMD Byte
      cmb = getc();        // Get Command Byte
      csown = mb;
      switch(cmb)
      {
        case ACPB :
          dat1 = getc();       // P = Byte*110/8
          dat2 = getc();       // temperature
          cs  = getc();
          csown += cmb;
          csown += dat1;
          csown += dat2;
          if(csown == cs)
          {
            acp = dat1;
            t   = dat2;
          }
          break;
        case SUB  :
          dat1 = getc();       // Solar U = Byte*4/3
          dat2 = getc();       // Netz U
          cs = getc();
          csown += cmb;
          csown += dat1;
          csown += dat2;
          if(csown == cs)
          {
            su = dat1;
            nu = dat2;
          }
          break;
        case WRSB :
          dat1 = getc();       // Wechselrichter Status
               getc();        // byte not used
          cs = getc();
          csown += cmb;
          csown += dat1;
          csown += FB;
          if (csown == cs)
            wrstatus = dat1;
          break;
        case WW   :
          dat1  = getc();       //
          dat2  = getc();       //
          cs  = getc() ;     
          csown += cmb;
          csown += dat1;
          csown += dat2;
          if(csown == cs)
          {
            wl = dat1;
            wh = dat2;
          }
          break;
        default :
          ;
          break;
      }
      
      p   = (long)acp;
      p   = p * 110;
      p   = p / 8;
      
      u   = (long) su;
      u   = u * 4;
      u   = u / 3;
     
      w   = wh;
      w   = w << 8;
      w   = w | wl; 
      
      tt  = (long) t;   // Temperatur T = (Byte -180)*3/2
      tt  = tt - 180;
      tt  = tt * 3;
      tt  = tt / 2;
 
      
      if (csown == cs)
      {
        if(!wrstatus)
        {
          output_low(PIN_A1);     // Clear Bit-Error LED
          lcd_gotoxy(1,1);
          printf(lcd_putc,"W=%5luWh",w);
          lcd_gotoxy(1,2);
          printf(lcd_putc,"P=%4luW Us=%3luV",p ,u);
          lcd_gotoxy(17,1);
          printf(lcd_putc,"T=%2luC  Un=%3uV",tt,nu); 
          lcd_gotoxy(17,2);
          printf(lcd_putc,"(c)M.Stehr 2002");
        }
        else
        {
          printf(lcd_putc,"\fW=%luWh",w); 
          printf(lcd_putc,"\nWR ist AUS :-("); 
        }
      }
      else
      {
        output_high(PIN_A1);   // Set Bit-Error LED  
      }
   }
   while(TRUE);
}

	


